@@ -1,6 +1,5 @@
 /*
  * Copyright (c) 2005-2011 Alon Bar-Lev <alon.barlev@gmail.com>
- * All rights reserved.
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -66,6 +65,7 @@
 #include "_pkcs11h-core.h"
 #include "_pkcs11h-session.h"
 #include "_pkcs11h-slotevent.h"
+#include "_pkcs11h-openssl.h"
 
 /*======================================================================*
  * COMMON INTERNAL INTERFACE
@@ -243,6 +243,9 @@ pkcs11h_getFeatures (void) {
 #if defined(ENABLE_PKCS11H_ENGINE_WIN32)
 		PKCS11H_FEATURE_MASK_ENGINE_CRYPTO_WIN32 |
 #endif
+#if defined(ENABLE_PKCS11H_ENGINE_MBEDTLS)
+		PKCS11H_FEATURE_MASK_ENGINE_CRYPTO_MBEDTLS |
+#endif
 #if defined(ENABLE_PKCS11H_DEBUG)
 		PKCS11H_FEATURE_MASK_DEBUG |
 #endif
@@ -337,6 +340,18 @@ pkcs11h_initialize (void) {
 	data->max_retries = _PKCS11H_DEFAULT_MAX_LOGIN_RETRY;
 	data->allow_protected_auth = TRUE;
 	data->pin_cache_period = _PKCS11H_DEFAULT_PIN_CACHE_PERIOD;
+
+#if defined(ENABLE_PKCS11H_OPENSSL)
+	_PKCS11H_DEBUG (
+		PKCS11H_LOG_DEBUG1,
+		"PKCS#11: Initializing openssl"
+	);
+
+	if (!_pkcs11h_openssl_initialize()) {
+		goto cleanup;
+	}
+#endif
+
 	data->initialized = TRUE;
 
 	_g_pkcs11h_data = data;
@@ -361,7 +376,7 @@ cleanup:
 			has_mutex_cache = FALSE;
 		}
 		if (has_mutex_session) {
-			_pkcs11h_threading_mutexFree (&data->mutexes.session); 
+			_pkcs11h_threading_mutexFree (&data->mutexes.session);
 			has_mutex_session = FALSE;
 		}
 #endif
@@ -390,6 +405,14 @@ pkcs11h_terminate (void) {
 	if (_g_pkcs11h_data != NULL) {
 		_pkcs11h_provider_t current_provider = NULL;
 
+#if defined(ENABLE_PKCS11H_OPENSSL)
+		_PKCS11H_DEBUG (
+			PKCS11H_LOG_DEBUG1,
+			"PKCS#11: Terminating openssl"
+		);
+		_pkcs11h_openssl_terminate();
+#endif
+
 		_PKCS11H_DEBUG (
 			PKCS11H_LOG_DEBUG1,
 			"PKCS#11: Removing providers"
@@ -461,7 +484,7 @@ pkcs11h_terminate (void) {
 			PKCS11H_LOG_DEBUG1,
 			"PKCS#11: Marking as uninitialized"
 		);
-		
+
 		_g_pkcs11h_data->initialized = FALSE;
 
 		while (_g_pkcs11h_data->providers != NULL) {
@@ -472,9 +495,9 @@ pkcs11h_terminate (void) {
 		}
 
 #if defined(ENABLE_PKCS11H_THREADING)
-		_pkcs11h_threading_mutexFree (&_g_pkcs11h_data->mutexes.global); 
+		_pkcs11h_threading_mutexFree (&_g_pkcs11h_data->mutexes.global);
 		_pkcs11h_threading_mutexFree (&_g_pkcs11h_data->mutexes.cache);
-		_pkcs11h_threading_mutexFree (&_g_pkcs11h_data->mutexes.session); 
+		_pkcs11h_threading_mutexFree (&_g_pkcs11h_data->mutexes.session);
 #endif
 
 		_g_pkcs11h_crypto_engine.uninitialize (_g_pkcs11h_crypto_engine.global_data);
@@ -707,7 +730,7 @@ pkcs11h_addProvider (
 	provider->slot_event_method = slot_event_method;
 	provider->slot_poll_interval = slot_poll_interval;
 	provider->cert_is_private = cert_is_private;
-		
+
 #if defined(_WIN32)
 	provider->handle = LoadLibraryA (provider_location);
 #else
@@ -733,7 +756,7 @@ pkcs11h_addProvider (
 		"C_GetFunctionList"
 	);
 	memmove (
-		&gfl, 
+		&gfl,
 		&p,
 		sizeof (void *)
 	);
@@ -918,7 +941,7 @@ pkcs11h_removeProvider (
 
 #if defined(ENABLE_PKCS11H_SLOTEVENT)
 	_pkcs11h_slotevent_notify ();
-	
+
 	/*
 	 * Wait until manager join this thread
 	 * this happens saldom so I can poll
@@ -944,7 +967,7 @@ pkcs11h_removeProvider (
 	rv = CKR_OK;
 
 cleanup:
-	
+
 #if defined(ENABLE_PKCS11H_THREADING)
 	for (
 		current_session = _g_pkcs11h_data->sessions;
@@ -1130,7 +1153,7 @@ _pkcs11h_log (
 	if (
 		_g_pkcs11h_data != NULL &&
 		_g_pkcs11h_data->initialized
-	) { 
+	) {
 		if (__PKCS11H_MSG_LEVEL_TEST (flags)) {
 			if (_g_pkcs11h_data->hooks.log == NULL) {
 				__pkcs11h_hooks_default_log (
@@ -1222,7 +1245,7 @@ __pkcs11h_hooks_default_pin_prompt (
 		user_data,
 		token->display
 	);
-	
+
 	return FALSE;
 }
 
